%A demo of the heat equation 
%Author: Jeff Calder, 2016

N = 1000; %Number of grid points
T = 250000; %Number of iterations
k = 100; %Display new graph every k iterations
h = 1/(N-1); %Grid spacing
dt = h^2/3; %Time step for stability

%Initial condition
x = 0:1/(N-1):1;
u0 = floor(sin(17*x)) + 1; %Square wave initial condition
u = u0;
y_min = -0.1; %Plot axes
y_max = 1.1;

%Draw initial plot
plot(x,u,'linewidth',2);
ylim([y_min y_max]);
title('Heat diffusion');
xlabel('Position');
ylabel('Temperature');
set(gca,'fontsize',15);
drawnow;
pause; %Pause until user clicks

%Main loop for solving PDE
for i=1:T

   %Draw plot every k iterations
   if mod(i,k) == 0
      plot(x,u,'linewidth',2);
      ylim([y_min y_max]);
      title('Heat diffusion');
      xlabel('Position');
      ylabel('Temperature');
      set(gca,'fontsize',15);
      drawnow;
   end

   %Note: Boundary conditions are encoded into these lines
   %      For a rod with insulated ends, use
   u = u + dt*(u([2:end,end-1]) - 2*u  + u([2,1:(end-1)]))/h^2;

   %For rod with Dirichlet boundary conditions use
   %a = 1; u(0,t) : Left boundary condition
   %b = 0; u(1,t) : Right boundary condition
   %u = u + dt*([u([2:end]),b] - 2*u  + [a,u([1:end-1])])/h^2;

   if i > T/4
      k = 5000; %Speed it up!
   end
end


%Plot steady state: Set up for boundary conditions u(0)=0 and u(1)=1.
plot(x,ones(1,length(x))*sum(u0)*h,'color','red','linewidth',2); %Use this line for rod with insulated ends
ylim([y_min y_max]);
title('Heat diffusion');
xlabel('Position');
ylabel('Temperature');
set(gca,'fontsize',15);
pause;

T = 10000;
k = 1; %Display new graph every k iterations
h = 1/(N-1); %Grid spacing
dt = h^2/3; %Time step for stability

%Initial condition
x = 0:1/(N-1):1;
u0 = sin(17*x) + 1 + rand(size(x)); 
u = u0;
y_min = 0; %Plot axes
y_max = 3;

plot(x,u);
ylim([y_min y_max]);
pause
hold;

%Main loop for solving PDE
for i=1:T
   u = u + dt*(u([2:end,end-1]) - 2*u  + u([2,1:(end-1)]))/h^2;

   if mod(i,k)==0
      hold;
      plot(x,u0,'LineWidth',0.1);
      hold;
      plot(x,u,'red','LineWidth',2);
      ylim([y_min y_max]);
      drawnow
      pause(0.05);
   end
end

plot(x,u);
ylim([y_min y_max]);



