%A demo of the wave equation
%Author: Jeff Calder, 2016

N = 1000; %Number of grid points
T = 200000; %Number of iterations
k = 300; %Draw graph every k iterations
h = 1/(N-1); %Grid spacing
dt = h/50; %Time step


%Initial condition
x = 0:1/(N-1):1;
m = 10; %Slope of initial condition
u0 = max(min(m*x + 1-m/2,-m*x + 1+m/2),0); %Three finger pluck initial condition
y_max = 1; %Plot axes
y_min = -1;

%Smooth out initial condition with heat equation
for i=1:500
   u0 = u0 + (u0([2:end,end-1]) - 2*u0  + u0([2,1:end-1]))/3;
end

u1 = u0; %Initialize values of u at previous two time steps
u2 = u0;

%Draw initial plot
plot(x,u1,'linewidth',2);
ylim([y_min y_max]);
title('Wave Propagation');
xlabel('Position');
ylabel('Displacement');
set(gca,'fontsize',15);
drawnow;
pause; %Pause until user clicks

%Main loop for solving PDE
for i=1:T

   %Draw plot every k iterations
   if mod(i,k) == 0
      plot(x,u1,'linewidth',2);
      ylim([y_min y_max]);
      title('Wave Propagation');
      xlabel('Position');
      ylabel('Displacement');
      set(gca,'fontsize',15);
      drawnow;
   end

   %Update solution of PDE
   %u = 2*u1 - u2 + dt^2*(u1([2:end,end-1]) - 2*u1  + u1([2,1:end-1]))/h^2;
   u = 2*u1 - u2 + dt^2*([u1(2:end),0] - 2*u1  + [0,u1(1:end-1)])/h^2;

   %Update previous history
   u2 = u1;
   u1 = u;
end

%Draw final state in red
plot(x,u1,'color','red','linewidth',2);
ylim([y_min y_max]);
title('Wave Propagation');
xlabel('Position');
ylabel('Displacement');
set(gca,'fontsize',15);
pause;



%Second example
u0 = zeros(1,N);
y_min = -1.1;
y_max = 1.1;
k=400;
T = 300000;

u1 = u0;
u2 = u0;

%Draw initial plot
plot(x,u1,'linewidth',2);
ylim([y_min y_max]);
title('Wave Propagation');
xlabel('Position');
ylabel('Displacement');
set(gca,'fontsize',15);
drawnow;
pause;

%Main loop for solving PDE
for i=1:T

   %Draw plot every k iterations
   if mod(i,k) == 0
      plot(x,u1,'linewidth',2);
      ylim([y_min y_max]);
      title('Wave Propagation');
      xlabel('Position');
      ylabel('Displacement');
      set(gca,'fontsize',15);
      drawnow;
   end

   %Update solution of PDE
   u = 2*u1 - u2 + dt^2*([u1([2:end]),0.5*max(sin(2*i/T*pi),0)*sin(30*i*dt)] - 2*u1  + [0,u1([1:end-1])])/h^2;
   u2 = u1;
   u1 = u;
end

%Draw final state in red
plot(x,u1,'color','red','linewidth',2);
ylim([y_min y_max]);
title('Wave Propagation');
xlabel('Position');
ylabel('Displacement');
set(gca,'fontsize',15);
pause;

